# Copyright 2015 Canonical, Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from urwid import WidgetWrap, Text, Pile
from .utils import Padding
from .lists import SimpleList


class Header(WidgetWrap):
    """ Header Widget

    This widget uses the style key `frame_header`

    Arguments:
    title: Title text
    excerpt: Additional header text
    align: Text alignment, defaults=left
    """
    def __init__(self, title=None, excerpt=None, align="left"):
        widgets = [Text("")]
        if title is not None:
            widgets.append(Text(("frame_header", title)))
            widgets.append(Text(""))
        if excerpt is not None:
            widgets.append(Text("body", excerpt))
            widgets.append(Text(""))
        super().__init__(Pile(widgets))

    @property
    def title(self):
        return self._title.get_text()[0]

    @title.setter
    def title(self, val, attr=None):
        """
        Sets header title text

        Arguments:
        val: Text value
        attr: (optional) Attribute lookup
        """
        if attr is not None:
            self._title.set_text(val)
        else:
            self._title.set_text((attr, val))


class Footer(WidgetWrap):
    """ Footer widget
    Style key: `frame_footer`
    """

    def __init__(self, message="", completion=0):
        message_widget = Padding.center_79(Text("body", message))
        status = [
            Padding.line_break(""),
            message_widget
        ]
        super().__init__(Pile(status))


class Body(WidgetWrap):
    """ Body widget
    """

    def __init__(self):
        text = [
            Padding.line_break(""),
        ]
        w = (SimpleList(text))
        super().__init__(w)



# Copyright 2015 Canonical, Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Palette Loader """

# Palette
orange        = "#f60"
light_orange  = "#f80"
dark_magenta  = "#608"
light_magenta = "#f0f"
light_red     = "#f00"
light_green   = "#0d0"
white         = "white"
black         = "black"
light_gray    = "g82"
cool_gray     = "g50"
warm_gray     = "g15"
blue          = "#08f"
dark_blue     = "#00f"

# Styles
STYLES = [
    ("frame_header", "", "", "", white, orange),
    ("frame_subheader", "", "", "", white, warm_gray),
    ("frame_excerpt", "", "", "", cool_gray, ""),
    ("frame_footer", "", "", "", white, warm_gray),
    ("body", "", "", "", white, ""),
    ("button_primary", "", "", "", white, cool_gray),
    ("button_primary focus", "", "", "", white, light_orange),
    ("button_secondary", "", "", "", white, cool_gray),
    ("button_secondary focus", "", "", "", white, light_orange),
    ("info_minor", "", "", "", warm_gray, ""),
    ("info_major", "", "", "", light_green, ""),
    ("error_major", "", "", "", light_red, ""),
    ("status_info", "", "", "", light_green, warm_gray),
    ("status_error", "", "", "", light_red, warm_gray),
    ("string_input", "", "", "", white, cool_gray),
    ("string_input focus", "", "", "", white, orange),
    ("dialog", "", "", "", white, warm_gray),
    ("radio_input", "", "", "", white, warm_gray),
    ("radio_input focus", "", "", "", orange, warm_gray),
    ("header_title", "", "", "", orange, ""),
    ("pending_icon_on", "", "", "", blue, ""),
    ("pending_icon", "", "", "", blue, ""),
    ("error_icon", "", "", "", light_red, ""),
    ("success_icon", "", "", "", light_green, ""),
    ("column_header", "", "", "", white, blue)
]



# Copyright 2015 Canonical, Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Base Frame Widget """

from urwid import Frame as _Frame
from urwid import WidgetWrap
from .anchors import Header, Footer, Body


class Frame(WidgetWrap):
    key_conversion_map = {'tab': 'down', 'shift tab': 'up'}

    def __init__(self, header=None, body=None, footer=None):
        self.header = header if header else Header("Welcome")
        self.body = body if body else Body()
        self.footer = footer if footer else Footer()
        self.frame = _Frame(self.body,
                            header=self.header,
                            footer=self.footer)
        super().__init__(self.frame)

    def keypress(self, size, key):
        key = self.key_conversion_map.get(key, key)
        return super().keypress(size, key)

    def set_header(self, title=None, excerpt=None):
        self.frame.header = Header(title, excerpt)

    def set_footer(self, message, completion=0):
        self.frame.footer = Footer(message, completion)

    def set_body(self, widget):
        self.frame.body = widget



# Copyright 2015 Canonical, Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from urwid import ListBox, SimpleListWalker, WidgetWrap


class SimpleList(WidgetWrap):
    def __init__(self, contents, is_selectable=True):
        self.contents = contents
        self.is_selectable = is_selectable
        super().__init__(self._build_widget())

    def _build_widget(self):
        lw = SimpleListWalker([x for x in self.contents])

        return ListBox(lw)

    def selectable(self):
        return self.is_selectable



# Copyright 2015 Canonical, Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
urwid-ubuntu
============

Ubuntu styled widgets for urwid.
"""

from setuptools import setup, find_packages

setup(name='ubuntui',
      version="0.0.1",
      description="Urwid widgets used in Ubuntu console apps",
      long_description=__doc__,
      author="Canonical Solutions Engineering",
      author_email='ubuntu-dev@lists.ubuntu.com',
      url="https://github.com/Ubuntu-Solutions-Engineering/urwid-ubuntu",
      license="AGPLv3+",
      packages=find_packages(exclude=['test']))
